@@ -15,12 +15,18 @@
 # You should have received a copy of the GNU General Public License along with
 # tfaip. If not, see http://www.gnu.org/licenses/.
 # ==============================================================================
+import os
+import tempfile
 import unittest
 
 from tensorflow.python.keras.backend import clear_session
 
 from examples.text.finetuningbert.scenario import FTBertScenario
-from test.util.training import single_train_iter
+from tfaip.util.testing.training import single_train_iter
+from tfaip.util.testing.workdir import call_in_root
+
+this_dir = os.path.dirname(os.path.realpath(__file__))
+root_dir = os.path.abspath(os.path.join(this_dir, "..", ".."))
 
 
 class FTBertTestScenario(FTBertScenario):
@@ -38,3 +44,15 @@ class TestFineTuningBert(unittest.TestCase):
 
     def test_single_train_iter(self):
         single_train_iter(self, self.scenario, debug=False)
+
+    def test_cmd_line_of_readme(self):
+        # Keep this in sync with README.md
+        with tempfile.TemporaryDirectory() as ftbert_model:
+            for cmd in [
+                f"tfaip-train examples.text.finetuningbert --trainer.output_dir {ftbert_model}",
+            ]:
+                additional_args = []
+                if "tfaip-train" in cmd:
+                    additional_args = ["--trainer.epochs", "1", "--trainer.samples_per_epoch", "32"]
+
+                call_in_root(cmd.split(" ") + additional_args)
