@@ -16,7 +16,7 @@
 # tfaip. If not, see http://www.gnu.org/licenses/.
 # ==============================================================================
 """General definitions for the Pipeline: PipelineMode, Sample"""
-from typing import Any
+from typing import Any, Optional
 
 from tfaip.util.enum import StrEnum
 
@@ -36,10 +36,12 @@ GENERAL_PROCESSOR = {PipelineMode.TRAINING, PipelineMode.EVALUATION, PipelineMod
 class Sample:
     """
     Basic structure of a single (unbatched) sample which is used to process data.
-    The sample provides four fields (inputs, outputs, targets, and optionally meta) which must be set dependen on the
+    The sample provides four fields (inputs, outputs, targets, and optionally meta) which must be set dependent on the
     PipelineMode.
 
-    Meta should be a dict with arbitrary serializable data.
+    Meta should be a dict with arbitrary SERIALIZABLE data.
+    Inputs, outputs, targets have to be (arbitrarily) nested structures of numpy arrays. Furthermore, scalars are not supported, i.e.,
+    use np.asarray([0], dtype=np.int32) instead of np.asarray(0, dtype=np.int32)
 
     Call the provided methods `new_inputs`, `new_outputs`, `new_targets`, `new_meta`, or `new_invalid` to build and
     return a new (copied) sample after transformations were applied.
@@ -54,6 +56,16 @@ class Sample:
     def new_invalid(self):
         return Sample(meta=self.meta)
 
+    def is_valid(self, mode: Optional[PipelineMode] = None) -> bool:
+        if mode is None:
+            return not (self.inputs is None and self.outputs is None and self.targets is None)
+        if self.inputs is None and mode in INPUT_PROCESSOR:
+            return False
+        if self.targets is None and mode in TARGETS_PROCESSOR:
+            return False
+
+        return True
+
     def copy(self):
         return Sample(inputs=self.inputs, outputs=self.outputs, targets=self.targets, meta=self.meta)
 
