@@ -24,10 +24,9 @@ import tensorflow as tf
 from typeguard import typechecked
 
 from tfaip import ModelBaseParams
-from tfaip import Sample
+from tfaip import Sample, EXPORT_TENSORFLOW_1
 from tfaip.data.data import DataBase
 from tfaip.model.metric.count import Count
-from tfaip.model.metric.multi import MultiMetricDefinition
 from tfaip.model.tensorboardwriter import TensorboardWriter
 from tfaip.util.tftyping import AnyTensor
 
@@ -40,9 +39,7 @@ TMP = TypeVar("TMP", bound=ModelBaseParams)
 
 
 class ModelBase(Generic[TMP], ABC):
-    """
-    The ModelBase class provides the implementation of the keras Model, its losses and metrics.
-    """
+    """The ModelBase class provides the implementation of the keras Model, its losses and metrics."""
 
     @classmethod
     def params_cls(cls) -> Type[TMP]:
@@ -68,8 +65,11 @@ class ModelBase(Generic[TMP], ABC):
         }
 
     def __init__(self, params: TMP, **kwargs):
+        assert len(kwargs) == 0, f"Not all kwargs processed by subclasses: {kwargs}"
         self._params: TMP = params
         self._count_metric = Count()
+        self.tensorboard_writer_metrics = []
+        self.metrics = None
 
     @staticmethod
     def root_graph_cls() -> Type["RootGraph"]:
@@ -114,10 +114,6 @@ class ModelBase(Generic[TMP], ABC):
         # Override this function
         return []
 
-    def _multi_metric(self) -> List[MultiMetricDefinition]:
-        # Override this function
-        return []
-
     @typechecked
     def sample_weights(self, inputs: Dict[str, AnyTensor], targets: Dict[str, AnyTensor]) -> Dict[str, Any]:
         """The weights of the samples. The output key must match the respective metric, extended_metric, or loss name.
@@ -187,6 +183,7 @@ class ModelBase(Generic[TMP], ABC):
         outputs: Dict[str, AnyTensor],
         targets: Dict[str, AnyTensor],
     ) -> Dict[str, tf.keras.Model]:
+        """Function to export additional (sub)-graphs alongside the `default` one."""
         eg = self._export_graphs(inputs, outputs, targets)
         if "default" not in eg:
             raise KeyError(f'Expected at least an export graph with label "default" in {eg}.')
@@ -203,6 +200,13 @@ class ModelBase(Generic[TMP], ABC):
         return {"default": tf.keras.Model(inputs=inputs, outputs=outputs)}
 
     def add_all_losses(self, model, inputs, targets, outputs):
+        """Add the losses, defined by `loss` and `loss_weights`.
+
+        Losses are summed up to form the final loss.
+        Each loss is added as a standalone metric.
+        """
+        aggregation = EXPORT_TENSORFLOW_1["metric_aggregation"]  # JAVA-Training Workaround, remove if better solution
+
         loss_weights = self.loss_weights() or {}
         total_loss = 0
         for name, loss in self.loss(inputs, targets, outputs).items():
@@ -210,40 +214,40 @@ class ModelBase(Generic[TMP], ABC):
             loss_v = tf.reduce_mean(loss)
             extra_loss = loss_v * loss_weight
             total_loss += extra_loss
-            model.add_metric(loss_v, name=name)
+            model.add_metric(loss_v, name=name, aggregation=aggregation)
 
         model.add_loss(total_loss)
-        model.add_metric(total_loss, name="loss/mean_epoch")
+        model.add_metric(total_loss, name="loss/mean_epoch", aggregation=aggregation)
 
     def add_all_metrics(self, model, inputs, targets, outputs):
+        """Add the metrics and their sample_weights to the model."""
         for metric_v in self.metric(inputs, targets, outputs):
             model.add_metric(metric_v)
 
         # add counter metric
         model.add_metric(self._count_metric(inputs, targets))
 
-        # convert multi metrics to simple metrics
-        if len(model.multi_metrics) > 0:
-            sample_weights = self.sample_weights(inputs, targets)
-            for v in model.multi_metrics:
-                for c in v.metric.children:
-                    model.add_metric(c(targets[v.target], outputs[v.output], sample_weights.get(c.name, None)))
-                model.add_metric(
-                    v.metric(targets[v.target], outputs[v.output], sample_weights.get(v.metric.name, None))
-                )
-
         if len(model.target_output_metrics) > 0:
             sample_weights = self.sample_weights(inputs, targets)
             for t, o, m in model.target_output_metrics:
                 model.add_metric(m(targets[t], outputs[o], sample_weights.get(m.name, None)))
 
     def pre_proc_targets(self, inputs, targets):
+        """Method to be called on the targets before outputs are computed."""
+        del inputs
         return targets
 
     def post_proc_targets(self, inputs, targets, outputs):
+        """Method to be called on the targets after outputs are computed."""
+        del inputs
+        del outputs
         return targets
 
     def wrap_model_with_loss_and_metric(self, model, inputs, targets, outputs, with_losses=True, with_metrics=True):
+        """Adds all losses and metrics.
+
+        This function will apply post-processing on the targets.
+        """
         post_proc_targets = self.post_proc_targets(inputs, targets, outputs)
         if with_losses:
             self.add_all_losses(model, inputs, post_proc_targets, outputs)
@@ -251,7 +255,7 @@ class ModelBase(Generic[TMP], ABC):
             self.add_all_metrics(model, inputs, post_proc_targets, outputs)
 
 
-class TFAIPKerasModel(tf.keras.models.Model):
+class TFAIPKerasModel(tf.keras.Model):
     def call(self, inputs, training=None, mask=None):
         return super().call(inputs, training=training, mask=mask)
 
