@@ -17,13 +17,15 @@
 # ==============================================================================
 """Definition of the WarmStarter"""
 import logging
+import os
 import re
-from typing import List, Optional, NoReturn
+from typing import List, Optional, NoReturn, TypeVar, Generic
 
 import numpy as np
 import tensorflow as tf
 
 from tfaip import WarmStartParams
+from tfaip.util.typing import AnyNumpy
 
 logger = logging.getLogger(__name__)
 
@@ -40,18 +42,26 @@ def longest_common_startstr(strs: List[str]) -> str:
     return longest_common
 
 
-class WarmStarter:
+TWSP = TypeVar("TWSP", bound=WarmStartParams)
+
+
+class WarmStarter(Generic[TWSP]):
     """The WarmStarter handles the loading of a pretrained model an applies the weights to the current one.
 
     See WarmStartParams for configuration. Both SavedModels and Checkpoints are supported.
     """
 
-    def __init__(self, params: WarmStartParams):
+    def __init__(self, params: TWSP, **kwargs):
+        assert len(kwargs) == 0, f"Not all kwargs processed by subclasses: {kwargs}"
         self._params = params
 
         if (params.exclude or params.include) and params.allow_partial:
             raise ValueError("Allow partial is only allowed if neither exclude not include is specified")
 
+    @property
+    def params(self) -> TWSP:
+        return self._params
+
     def _trim(self, names: List[str]) -> List[str]:
         if self._params.trim_graph_name:
             longest_common = longest_common_startstr([n for n in names if "print_limit" not in n])
@@ -100,13 +110,14 @@ class WarmStarter:
         # 3. Load weights and assign (only works if the model is identical), if failed -> 4
         # 4. Load weights by name -> 5
         # 5. Apply renaming rules and match weights
+        abs_model_path = os.path.abspath(os.path.expanduser(self._params.model))
         try:
             # 1. Load model
-            src_model = tf.keras.models.load_model(self._params.model, compile=False, custom_objects=custom_objects)
+            src_model = tf.keras.models.load_model(abs_model_path, compile=False, custom_objects=custom_objects)
         except OSError:
             # 2. load as checkpoint, then go to 5.
-            logger.debug(f"Could not load '{self._params.model}' as saved model. Attempting to load as a checkpoint.")
-            ckpt = tf.train.load_checkpoint(self._params.model)
+            logger.debug(f"Could not load '{abs_model_path}' as saved model. Attempting to load as a checkpoint.")
+            ckpt = tf.train.load_checkpoint(abs_model_path)
             name_shapes = ckpt.get_variable_to_shape_map()
             model_to_load_var_names = name_shapes.keys()
 
@@ -175,14 +186,6 @@ class WarmStarter:
             )
         }
         target_var_name_to_trainable_name = {v: k for k, v in trainable_name_to_target_var_name.items()}
-        logger.info(f"Warm-starting from {self._params.model}")
-        try:
-            # First try to reinstantiate the model, and apply the renamings on the weights,
-            # if this fails, load the weights as checkpoints, apply additional renamings, and then try to match
-            # the source and target weights
-            model = tf.keras.models.load_model(self._params.model, compile=False, custom_objects=custom_objects)
-        except OSError:
-            logger.debug(f"Could not load '{self._params.model}' as saved model. Attempting to load as a checkpoint.")
 
         # Filter the params and validate
         names_target = set(trainable_name_to_target_var_name.keys())
@@ -202,6 +205,7 @@ class WarmStarter:
         elif self._params.allow_partial:
             names_to_load = names_target.intersection(names_loaded)
         else:
+            names_to_load = names_target
             diff_target = names_loaded.difference(names_target)
             diff_load = names_target.difference(names_loaded)
             if len(diff_target) > 0 or len(diff_load) > 0:
@@ -210,7 +214,6 @@ class WarmStarter:
                     f"\nUse allow_partial to allow partial loading"
                 )
 
-        names_to_load = names_target
         new_weights = []
         warm_weights_names = []
         cold_weights_names = []
@@ -220,7 +223,11 @@ class WarmStarter:
             trainable_name = target_var_name_to_trainable_name.get(name, None)  # None == not existing
             if trainable_name in names_to_load:
                 warm_weights_names.append(name)
-                new_weights.append(all_loaded_weights[trainable_name_to_loaded_var_name[trainable_name]])
+                new_weights.append(
+                    self._transform_weight(
+                        all_loaded_weights[trainable_name_to_loaded_var_name[trainable_name]], trainable_name, name
+                    )
+                )
             else:
                 if name in all_trainable_target_weights:
                     cold_weights_names.append(name)
@@ -250,3 +257,6 @@ class WarmStarter:
             new_weights: New weights of the model
         """
         target_model.set_weights(new_weights)
+
+    def _transform_weight(self, weight: AnyNumpy, trainable_name: str, name: str) -> AnyNumpy:
+        return weight
