@@ -20,8 +20,11 @@ import os
 import logging
 
 # Initialize logging
+import re
 import sys
 
+import numpy as np
+
 FORMAT = "{levelname:<8s} {asctime} {name:>30.30s}: {message}"
 formatter = logging.Formatter(FORMAT, style="{")
 TFAIP_LOG_LEVEL = getattr(logging, os.environ.get("TFAIP_LOG_LEVEL", "INFO").upper())
@@ -70,7 +73,32 @@ class WriteToLogFile:
         this_logger.info(f"Logging to '{self.filename}'")
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        handle_exception(exc_type, exc_val, exc_tb)  # Log exception before log gets closed
+        if exc_type is not None:
+            handle_exception(exc_type, exc_val, exc_tb)  # Log exception before log gets closed
         logging.getLogger().removeHandler(self.file_handler)
         self.file_handler.flush()
         self.file_handler.close()
+
+
+class ParseLogFile:
+    def __init__(self, log_dir: str, log_name="train.log"):
+        assert log_dir is not None
+        self.filename = os.path.join(log_dir, log_name)
+
+    def get_metrics(self) -> dict:
+        if not os.path.isfile(self.filename):
+            return dict()
+        with open(self.filename, "r") as f:
+            # get last epoch line index
+            res_lines = []
+            for [i, line] in enumerate(f, 0):
+                if "Results of epoch" in line:
+                    res_lines.append([i, line])
+
+            assert len(res_lines) > 0, "Found no result lines in log-file"
+            last_res = res_lines[-1][1]
+
+            # parse losses and metrics
+            metrics_re = re.findall(r"([\w/]+): ([-0-9.]+)", last_res)
+            metrics = {ele[0]: ele[1] for ele in metrics_re}
+        return metrics
