@@ -26,17 +26,16 @@ def shared_value_field(**kwargs):
     Usage in dataclass:
         param: multiprocessing.Value = shared_value_field(default_factory=lambda: mp_manager().Value('i', 0))
     """
-    assert 'default_factory' in kwargs
+    assert "default_factory" in kwargs
 
     def decode(x):
-        default = kwargs['default_factory']()
+        default = kwargs["default_factory"]()
         default.value = x
         return default
 
-    kwargs['metadata'] = {
-        **kwargs.get('metadata', {}),
+    kwargs["metadata"] = {
+        **kwargs.get("metadata", {}),
         **dataclasses_json.config(encoder=lambda x: x.value, decoder=decode),
-        **pai_meta(mode='ignore'),
+        **pai_meta(mode="ignore"),
     }
     return field(**kwargs)
-
